// ***************************************************************************
// LICENSING:  This is free, open source software, licensed under
//             Version 2 of the GNU General Public License (GPL).
//
// In particular, this means that distribution of this software in a binary
// format, e.g. as compiled in as part of a .ex4 format, must be accompanied
// by the non-obfuscated source code of both this file, AND the .mq4 source
// files which it is compiled with, or you must make such files available at
// no charge to binary recipients. If you do not agree with such terms you
// must not use this code.  Detailed terms of the GPL are widely available
// on the Internet.  The Library GPL (LGPL) was intentionally not used,
// therefore the source code of files which link to this are subject to
// terms of the GPL if binaries made from them are publicly distributed or
// sold.
//
// ANY USE OF THIS CODE NOT CONFORMING TO THIS LICENSE MUST FIRST RECEIVE
// PRIOR AUTHORIZATION FROM THE AUTHOR(S).  ANY COMMERCIAL USE MUST FIRST
// OBTAIN A COMMERCIAL LICENSE FROM THE AUTHOR(S).
// ***************************************************************************
//
// This library is base upon the code found at:
// http://www.forexfactory.com/showthread.php?t=19293
//
// Please be thanful to:
//  * Derk Wehler (derkwehler@gmail.com)
//  * Paul Hampton-Smith
//  * Mike Nguyen
//  * Fai Hatena
// And many others who have contributed to this code
//
// ***************************************************************************
// HISTORY:
//
// v1 20140103, Sergien:-
//   - Stand alone version of functions
// v2 20140207, Sergien:-
//   - Modified to Build 600
//   - Added code by lukins (http://codebase.mql4.com/9240) to get the Internet data
//
// ***************************************************************************

#property copyright "Copyright 2013, SergienSergien"
#property link "sergiensergien@gmail.com"
#property library

#define NEWS_MODULE_NAME "LibNEWS"

extern string  __MODULE_NEWS_FILTER__           = "____________________________________________________";
extern string  __MODULE_NEWS_FILTER_title__     = "=== NEWS FILTER SECTION ============================";
extern bool    NEWS_UseFilter                   = false;
extern string	__events_selection__			      = "--- NEWS EVENTS SELECTION --------------------------";
extern int   	NEWS_SelectEventsWithCriticity   = 3;       // 0: Everything; 1: Low; 2: Medium; 3: High
extern string  __timezones_configuration__		= "--- SOURCE AND NEWS_TIMEZONES CONFIGURATION ---";
bool		      NEWS_AutoNEWS_GMTZones			   = false;   // Set this to FALSE. It does not work ... yet! 
extern int		NEWS_LocalNEWS_GMTZone			   = 1;       // NEWS_GMT+X
extern int		NEWS_BrokerNEWS_GMTZone				= 2;       // NEWS_GMT+Y
extern bool		NEWS_UseDailyFXAsSource			   = true;    // Set this to TRUE when backtesting
extern bool 	NEWS_UseGlobalTimeReference 		= true;    // Set this to TRUE when back
extern string  __events_window__			         = "--- TRADE WINDOW FOR EVENTS ---";
extern int		NEWS_MinutesBeforeNews			   = 180;	  // Set to -1 for no Alert
extern int		NEWS_MinutesAfterNews			   = 60;		  // Set to -1 for no Alert


#define  NEWS_TITLE		   0
#define  NEWS_COUNTRY	   1
#define  NEWS_DATE		   2
#define  NEWS_TIME		   3
#define  NEWS_IMPACT		   4
#define  NEWS_FORECAST	   5
#define  NEWS_PREVIOUS	   6
#define  NEWS_GMT			   7

#define NEWS_NEWS_IMPACT_HIGH     3 
#define NEWS_NEWS_IMPACT_MEDIUM   2
#define NEWS_NEWS_IMPACT_LOW      1
#define NEWS_NEWS_IMPACT_UNKNOWN  0


int      DebugLevel              = 0;

int      RefreshMin              = 0;
int      RefreshHour             = 0;

int      AllNewsIdx              = 0;
string   allData[20000][8];

bool     SERVER_To_UTC_Auto      = false;
datetime LastUpdateTime	         = 0;

int      ServerNEWS_GMTZone		= 0;
int      LocalNEWS_GMTZone 		= 0;

bool		UseExternalTimeReference = false;
int      WeeksLookBack           = 0;

//=================================================================================================
//=================================================================================================
//==============================   NEWS Library Functions   ======================================
//=================================================================================================
//=================================================================================================

void NEWSInitLibrary() {
   MathSrand (TimeLocal());               //random number
	RefreshMin  = (MathRand() % 60);
	RefreshHour = (MathRand() % 6);

	Print (NEWS_MODULE_NAME, ": TimeUTC : ", TimeToStr(TimeLocalToGMT(AltTimeLocal())));

	// SERGIEN: We delete the global variable as, if an EA is testing, it will set it anew.
	GlobalVariableDel("_TimeReference");
}

void NEWSDeinitLibrary() {
	// SERGIEN: We delete the global variable as, if an EA is testing, it will set it anew.
	GlobalVariableDel("_TimeReference");
}

void NEWSSetTimeOffset (bool autoZone, int LocalZone, int ServerZone) {
	SERVER_To_UTC_Auto = autoZone;
   LocalNEWS_GMTZone = LocalZone;
	ServerNEWS_GMTZone = ServerZone;
}

void NEWSSetTimeReference (bool externalreference) {
	UseExternalTimeReference = externalreference;
}

void NEWSSetHistoryWeeksLookBack (int weeks = 0) {
	WeeksLookBack = weeks;
}

int NEWSGetTotalEvents () {
   return (AllNewsIdx);
}

bool NEWSGetEvent (int index, string& buffer[]) {
   if (index >= AllNewsIdx || ArraySize(buffer) < 8)
      return (false);
      
   for (int i = 0; i < 8; i++)
      buffer[i] = allData[index][i];
      
   return (true);
}

bool NEWSRefreshEvents (bool DailyFXPreferred = false) {
   bool ffXMLexists = false;

   // Set global counter to 0
	AllNewsIdx = 0;

   if(!DailyFXPreferred) {
		string ForexFactoryFileName = ForexFactoryGetXMLFileName(AltTimeLocal());
		ffXMLexists = ForexFactoryGetXMLFile (ForexFactoryFileName);
      if (ffXMLexists) {
         ForexFactoryReadXMLFile (ForexFactoryFileName);
         return (true);
      }
   } 
   
	// We try to get one get before;
	// if (DailyFXGetCSVFile(AltTimeLocal() - 10080))
	// 	DailyFXReadCSVFile(AltTimeLocal() - 10080); 
		
	// We try to get one get after if we are testing;
	// if (GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
	//	if(DailyFXGetCSVFile(AltTimeLocal() + 10080))
	//		DailyFXReadCSVFile(AltTimeLocal() + 10080); 
 
	if (!DailyFXGetCSVFile(AltTimeLocal()))
      return (false);
		
   DailyFXReadCSVFile(AltTimeLocal());
   
   return (true);
}

int NEWSNextEvent (int criticity, string& buffer[]) {
	datetime now = TimeLocalToGMT(AltTimeLocal());
	datetime showtime = 0;
	string 	event[8];
	int found = -1, impactfound = -1;
	
	for (int i = 0; i < NEWSGetTotalEvents(); i++) {
		NEWSGetEvent (i, event);
		int eventImpact = NEWSImpactToNumber(event[NEWS_IMPACT]);
		datetime eventNEWS_GMT = StrToTime(event[NEWS_GMT]);
		if (eventNEWS_GMT >= now && (eventNEWS_GMT < showtime || showtime == 0) && eventImpact >= criticity && eventImpact > impactfound) {
			showtime = eventNEWS_GMT;
			found = i;
			impactfound = eventImpact;
		}
	}

	if (found >= 0)
		NEWSGetEvent (found, buffer);

	return (found);
}

int NEWSPreviousEvent (int criticity, string& buffer[]) {
	datetime now = TimeLocalToGMT(AltTimeLocal());
	datetime showtime = 0;
	string 	event[8];
	int found = -1, impactfound = -1;
	
	for (int i = 0; i < NEWSGetTotalEvents(); i++) {
		NEWSGetEvent (i, event);
		int eventImpact = NEWSImpactToNumber(event[NEWS_IMPACT]);
		datetime eventNEWS_GMT = StrToTime(event[NEWS_GMT]);
		if (eventNEWS_GMT <= now && eventNEWS_GMT > showtime && eventImpact >= criticity && eventImpact > impactfound) {
			showtime = eventNEWS_GMT;
			found = i;
			impactfound = eventImpact;
		}
	}

	if (found >= 0)
		NEWSGetEvent (found, buffer);
	
	return (found);
}

bool NEWSDailyFXGetCSVFile (datetime requestedtime = 0) {
   return(DailyFXGetCSVFile(requestedtime));
}

//=================================================================================================
//=================================================================================================
//======================   ForexFactory Calendar Functions   ======================================
//=================================================================================================
//=================================================================================================

string	ffUrl = "http://forexfactory.com/ffcal_week_this.xml";
string 	sTags[7] = { "<title>", "<country>", "<date><![CDATA[", "<time><![CDATA[", "<impact><![CDATA[", "<forecast><![CDATA[", "<previous><![CDATA[" };
string 	eTags[7] = { "</title>", "</country>", "]]></date>", "]]></time>", "]]></impact>", "]]></forecast>", "]]></previous>" };
//old tag		   	   
//string 	sTags[7] = { "<title>", "<country>", "<date>", "<time>", "<impact>", "<forecast>", "<previous>" };
//string 	eTags[7] = { "</title>", "</country>", "</date>", "</time>", "</impact>", "</forecast>", "</previous>" };

string ForexFactoryGetXMLFileName(datetime askedtime = 0) {
//	return (Month() + "-" + Day() + "-" + Year() + "-" + Symbol() + Period() + "-" + "NEWS.xml");
	datetime theTime;
	if (askedtime == 0)
		theTime = AltTimeCurrent();
	else	
		theTime = askedtime;
	return (StringConcatenate("NEWS-", TimeYear(theTime), ".", TimeMonth(theTime), ".", TimeDay(theTime), ".xml"));
}  


bool ForexFactoryGetXMLFile(string xmlFileName) {
   string sData = "";
   bool NeedToGetFile = false;
   // First, check if the XML file already exists.  
	// If it does NOT, then we need to set a flag to go get it
	int xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ);

	// File does not exist if FileOpen return -1 or if GetLastError = ERR_CANNOT_OPEN_FILE (4103)
	if (xmlHandle >= 0)
		// Since file exists, close what we just opened
		FileClose(xmlHandle);
	else	
		NeedToGetFile = true;

	// Let's see if we are backtesting
	if (GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
		// Here we have a problem ... we cannot access an previous XML online
		if (NeedToGetFile)
			return (false);
		// But if we have it available we don't have to update it, so we return directly
		else
			return (true);


	//added by MN. Set this to false when using in another EA or Chart, so that the multiple 
	//instances of the indicator dont fight with each other
	// if (AllowWebUpdates) {
		// New method: Use global variables so that when put on multiple charts, it 
		// will not update overly often; only first time and every 4 hours
		// if (DebugLevel > 1)
		//	Print(GlobalVariableGet("LastUpdateTime") + " " + (AltTimeCurrent() - GlobalVariableGet("LastUpdateTime")));
			
   // if (NeedToGetFile || GlobalVariableCheck("LastUpdateTime") == false || 
   //               (AltTimeCurrent() - GlobalVariableGet("LastUpdateTime")) > 14400) {
	if (NeedToGetFile || (AltTimeCurrent() - LastUpdateTime) > 14400) {
			if (DebugLevel > 0)
				Print(NEWS_MODULE_NAME,": Grabbing Web, url = ", ffUrl);
	
			// THIS CALL WAS DONATED BY PAUL TO HELP FIX THE RESOURCE ERROR
			// GrabWeb(ffUrl, sData);
			sData = httpGET(ffUrl);
			if (DebugLevel > 0) {
				Print (NEWS_MODULE_NAME, ": Opening XML file...");
				Print (NEWS_MODULE_NAME, ": ", sData);
			}
         
			// Delete existing file
			FileDelete(xmlFileName);
			
			// Write the contents of the ForexFactory page to an .htm file
			// If it is still open from the above FileOpen call, close it.
			xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_WRITE);
			if (xmlHandle < 0) {
				if (DebugLevel > 0)
					Print(NEWS_MODULE_NAME, ": Can\'t open new xml file, the last error is ", GetLastError());
				return (false);
			}
			FileWriteString (xmlHandle, sData, StringLen(sData));
			FileClose (xmlHandle);
			if (DebugLevel > 0)
				Print(NEWS_MODULE_NAME, ": Wrote XML file...");

			// THIS BLOCK OF CODE DONATED BY WALLY TO FIX THE RESOURCE ERROR
			//--- Look for the end XML tag to ensure that a complete page was downloaded ---//
			int end = StringFind(sData, "</weeklyevents>", 0);
			if (end <= 0) {
				Alert(NEWS_MODULE_NAME + ": NEWS Error - Web page download was not complete!");
				return (false);
			}
			else
			// set global to time of last update
			//	GlobalVariableSet("LastUpdateTime", AltTimeCurrent());
				LastUpdateTime = AltTimeCurrent();
			//-------------------------------------------------------------------------------//
			
	}
	return (true);
}


int ForexFactoryReadXMLFile (string xmlFileName) {
   // Open the XML file
	int xmlHandle = FileOpen(xmlFileName, FILE_BIN|FILE_READ|FILE_ANSI);
	if (xmlHandle < 0) {
		Print (NEWS_MODULE_NAME, ": Can\'t open xml file: ", xmlFileName, ".  The last error is ", GetLastError());
		return (false);
	}
		
//BART --->
   string sData = "";
   while (StringLen(sData) < NormalizeDouble(FileSize(xmlHandle),0)){
	   sData = sData + FileReadString(xmlHandle, FileSize(xmlHandle)-StringLen(sData));				   
	}
/* BART	
   // Read in the whole XML file
	// Ave file length == ~7K, so 65536 should always read whole file
	sData = FileReadString(xmlHandle, 65536);	
   if ((int)StringLen(sData) < (int)FileSize(xmlHandle))
	   sData = sData + FileReadString(xmlHandle, FileSize(xmlHandle));	
/**/	
	// Because MT4 build 202 complained about too many files open and MT4 hung. Added by MN
	if (xmlHandle > 0)
		FileClose(xmlHandle);

	int BoEvent = 0;
	while (true)
	{
		BoEvent = StringFind(sData, "<event>", BoEvent);
		if (BoEvent == -1)
			break;
			
		BoEvent += 7;	
		int next = StringFind(sData, "</event>", BoEvent);
		if (next == -1)
			break;
	
		string myEvent = StringSubstrOld(sData, BoEvent, next - BoEvent);
		BoEvent = next;
		
		int begin = 0;
		for (int i = 0; i < 7; i++) {
			allData[AllNewsIdx][i] = "";
			next = StringFind(myEvent, sTags[i], begin);
			
			// Within this event, if tag not found, then it must be missing; skip it
			if (next == -1)
				continue;
			else {
				// We must have found the sTag okay...
				begin = next + StringLen (sTags[i]);			// Advance past the start tag
				int end = StringFind(myEvent, eTags[i], begin);	// Find start of end tag
				if (end > begin && end != -1)
				{
					// Get data between start and end tag
					allData[AllNewsIdx][i] = StringSubstrOld(myEvent, begin, end - begin);
				}
			}
		}
		allData[AllNewsIdx][NEWS_GMT] = MakeDateTime(allData[AllNewsIdx][NEWS_DATE], allData[AllNewsIdx][NEWS_TIME]);
		
		// Print("FOREX FACTORY : Title: ", allData[AllNewsIdx][NEWS_TITLE],
		//							" Country: ", allData[AllNewsIdx][NEWS_COUNTRY], 
		//							" Date: ", allData[AllNewsIdx][NEWS_DATE], 
		//							" Time: ", allData[AllNewsIdx][NEWS_TIME],
		//							" Impact: ", allData[AllNewsIdx][NEWS_IMPACT],
		//							" Forecast: ", allData[AllNewsIdx][NEWS_FORECAST], 
		//							" Previous: ", allData[AllNewsIdx][NEWS_PREVIOUS], 		
		//							" NEWS_GMT: ", allData[AllNewsIdx][NEWS_GMT]);		

		AllNewsIdx++;
	}

	// Print ("Loaded ", AllNewsIdx, " news from file ", xmlFileName);

   return (AllNewsIdx);
}

double NEWSImpactToNumber(string impact) {
	if (impact == "High")
		return (NEWS_NEWS_IMPACT_HIGH);
	if (impact == "Medium")
		return (NEWS_NEWS_IMPACT_MEDIUM);
	if (impact == "Low")
		return (NEWS_NEWS_IMPACT_LOW);
	else
		return (NEWS_NEWS_IMPACT_UNKNOWN);
}
   
string MakeDateTime(string strDate, string strTime) {
	// Print("Converting Forex Factory Time into Metatrader time..."); //added by MN
	// Converts forexfactory time & date into yyyy.mm.dd hh:mm
	int n1stDash = StringFind(strDate, "-");
	int n2ndDash = StringFind(strDate, "-", n1stDash+1);

	string strMonth = StringSubstrOld(strDate, 0, 2);
	string strDay = StringSubstrOld(strDate, 3, 2);
	string strYear = StringSubstrOld(strDate, 6, 4); 
//	strYear = "20" + strYear;
	
	int nTimeColonPos = StringFind(strTime, ":");
	string strHour = StringSubstrOld(strTime, 0, nTimeColonPos);
	string strMinute = StringSubstrOld(strTime, nTimeColonPos+1, 2);
	string strAM_PM = StringSubstrOld(strTime, StringLen(strTime)-2);

	int nHour24 = StrToInteger(strHour);
	if ((strAM_PM == "pm" || strAM_PM == "PM") && nHour24 != 12)
	{
		nHour24 += 12;
	}
	if ((strAM_PM == "am" || strAM_PM == "AM") && nHour24 == 12)
	{
		nHour24 = 0;
	}
 	string strHourPad = "";
	if (nHour24 < 10) 
		strHourPad = "0";

	return(StringConcatenate(strYear, ".", strMonth, ".", strDay, " ", strHourPad, nHour24, ":", strMinute));
}


//=================================================================================================
//=================================================================================================
//======================   ForexFactory Calendar Functions   ======================================
//=================================================================================================
//=================================================================================================
bool DailyFXGetCSVFile(datetime requestedtime = 0) {
	bool NeedToGetFile = true;
	string tmpData;

	datetime CurrentNEWS_GMT;
	if (requestedtime == 0)
		CurrentNEWS_GMT = TimeLocalToGMT(AltTimeLocal());
	else
		CurrentNEWS_GMT = TimeLocalToGMT(requestedtime);

	string file = DailyFXGetFileName(CurrentNEWS_GMT);  

	int Handle;
	Handle = FileOpen( file, FILE_BIN | FILE_READ );
	if ( Handle >= 0 ) {
		FileClose(Handle);
		return (true);
	}

	string sUrl = "http://www.dailyfx.com/files/" + file;
  
	// GrabWeb (sUrl, tmpData ); // Get HTTP Data..
   tmpData = httpGET (sUrl);

	Handle = FileOpen( file, FILE_BIN | FILE_WRITE );
	if ( Handle < 0 ) {
		Print (NEWS_MODULE_NAME, ": Can\'t open new  file, the last error is ", GetLastError() );
      return (false);
	} else {
      Print (NEWS_MODULE_NAME, ": Obtained web file :", sUrl);
	}
    
	FileWriteString( Handle, tmpData, StringLen( tmpData ) );
	FileClose( Handle );

	if( StringSubstrOld( tmpData, 0, 20 ) != "Date,Time,Time Zone," ) {
		Print( "[" + StringSubstrOld( tmpData, 0, 20 ) + "]" );
      tmpData = "";
      return (false);
	} 
  
	return (true);
}


//=================================================================================================
string DailyFXGetFileName( datetime tm ) {
  tm = tm - TimeDayOfWeek( tm ) * 24 * 60 * 60;
  string day = TimeDay( tm );
  string month = TimeMonth( tm );
  string year = TimeYear( tm );

  if( StringLen( day ) == 1 ) day = "0" + day;
  if( StringLen( month ) == 1 ) month = "0" + month;
  return( "Calendar-" + month + "-" + day + "-" + year + ".csv" );

}



//=================================================================================================
int DailyFXReadCSVFile(datetime requestedtime = 0) {
	datetime CurrentNEWS_GMT;
	
	if (requestedtime == 0)
		CurrentNEWS_GMT = TimeLocalToGMT(AltTimeLocal());
	else
		CurrentNEWS_GMT = TimeLocalToGMT(requestedtime);
    
   for (int i = WeeksLookBack + 1; i > 0; i--) {
	   string file = DailyFXGetFileName(CurrentNEWS_GMT - (i-1)*7*24*3600);
	   
   	int handle = FileOpen( file, FILE_CSV | FILE_READ, "," );
	   if ( handle < 0 ) {
		   Print( "Can\'t open ", file, ".. the last error is ", GetLastError() );
		   return( 0 );
	   }
  
	   string dmy;
	   // skip first line.
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );
	   dmy = FileReadString( handle );

	   while( !FileIsEnding( handle ) ) {
		   //Date	Time	TimeZone	Currency	Event	Importance	Actual	Forecast	Previous
		   allData[AllNewsIdx][NEWS_DATE]     = FileReadString( handle );
		   allData[AllNewsIdx][NEWS_TIME]     = FileReadString( handle );
		   dmy                   			= FileReadString( handle );
		   allData[AllNewsIdx][NEWS_COUNTRY]  = DailyFXtoUpper( FileReadString( handle ) );
		   string title = FileReadString (handle);
		   if (StringSubstrOld(title, 0, 1) == CharToStr(34)) {
			   while (StringSubstrOld(title, StringLen(title) - 1) != CharToStr(34))
   				title = title + FileReadString (handle); 
			   title = StringSubstrOld(title, 1, StringLen(title) - 2);
		   }

		   allData[AllNewsIdx][NEWS_TITLE]    = DailyFXDelSymbolHeader( title );
		   allData[AllNewsIdx][NEWS_IMPACT]   = FileReadString( handle );
		   dmy                   			     = FileReadString( handle );
		   allData[AllNewsIdx][NEWS_FORECAST] = FileReadString( handle );
		   allData[AllNewsIdx][NEWS_PREVIOUS] = FileReadString( handle );
		
		   string DateArray[3];
		   DailyFXExplode( allData[AllNewsIdx][NEWS_DATE], " ", DateArray );
		   allData[AllNewsIdx][NEWS_GMT] = TimeToStr(StrToTime(StringConcatenate( TimeYear( CurrentNEWS_GMT ), ".", DailyFXConvMonth( DateArray[1] ), ".", DateArray[2], " ", allData[AllNewsIdx][NEWS_TIME] )), TIME_DATE|TIME_MINUTES);

		   // Print("DailyFX : Title: ", allData[AllNewsIdx][NEWS_TITLE],
		   //							" Country: ", allData[AllNewsIdx][NEWS_COUNTRY], 
   		//							" Date: ", allData[AllNewsIdx][NEWS_DATE], 
   		//							" Time: ", allData[AllNewsIdx][NEWS_TIME],
   		//							" Impact: ", allData[AllNewsIdx][NEWS_IMPACT],
   		//							" Forecast: ", allData[AllNewsIdx][NEWS_FORECAST], 
   		//							" Previous: ", allData[AllNewsIdx][NEWS_PREVIOUS], 
   		//							" NEWS_GMT: ", allData[AllNewsIdx][NEWS_GMT]);		

		   AllNewsIdx++;
   	}
	   FileClose( handle );
   	// Print ("Loaded ", AllNewsIdx, " news from file ", file);
	}
	
   return (AllNewsIdx);
}

//=================================================================================================
string DailyFXDelSymbolHeader( string m ) {
  string Sym[] = {"AUD ", "JPY ", "NZD ", "CHF ", "USD ", "EUR ", "GBP ", "CAD ", "CNY "};
  for( int i = 0; i < ArraySize( Sym ); i++ ) {
    if ( StringSubstrOld( m, 0, 4 ) == Sym[i] ) {
      m = StringSubstrOld( m, 4 );
      return( m );
    }
  }
  return( m );
}
//=================================================================================================
int DailyFXConvMonth( string m ) {
  int mon = 0;
  string Mon[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
  for( int i = 0; i < ArraySize( Mon ); i++ )
    if( m == Mon[i] ) return( i + 1 );
  return(0);
}
//=================================================================================================
string DailyFXtoUpper( string s ) {
  int n = StringLen( s );
  for ( int i = 0; i < n; i++ ) {
    int c = StringGetChar( s, i );
    if ( c >= 'a' && c <= 'z' ) {
      c += 'A' - 'a';
      s = StringSetChar( s, i, c );
    }
  }

  return( s );
}

//=================================================================================================
int DailyFXExplode( string str, string delimiter, string& arr[] ) {
  int i = 0;
  int pos = StringFind( str, delimiter );
  while( pos != -1 ) {
    if( pos == 0 ) arr[i] = "";
    else arr[i] = StringSubstrOld( str, 0, pos );
    i++;
    str = StringSubstrOld( str, pos + StringLen( delimiter ) );
    pos = StringFind( str, delimiter );
    if( pos == -1 || str == "" ) break;
  }
  arr[i] = str;

  return( i + 1 );
}
//=================================================================================================




//=================================================================================================
//=================================================================================================
//====================================   GrabWeb Functions   ======================================
//=================================================================================================
//=================================================================================================
// Main Webscraping function
// ~~~~~~~~~~~~~~~~~~~~~~~~~
// bool GrabWeb(string strUrl, string& strWebPage)
// returns the text of any webpage. Returns false on timeout or other error
// 
// Parsing functions
// ~~~~~~~~~~~~~~~~~
// string GetData(string strWebPage, int nStart, string strLeftTag, string strRightTag, int& nPos)
// obtains the text between two tags found after nStart, and sets nPos to the end of the second tag
//
// void Goto(string strWebPage, int nStart, string strTag, int& nPos)
// Sets nPos to the end of the first tag found after nStart 

bool bWinInetDebug = false;

int hSession_IEType;
int hSession_Direct;
int Internet_Open_Type_Preconfig = 0;
int Internet_Open_Type_Direct = 1;
int Internet_Open_Type_Proxy = 3;
int Buffer_LEN = 80;

#import "wininet.dll"

#define INTERNET_FLAG_PRAGMA_NOCACHE    0x00000100 // Forces the request to be resolved by the origin server, even if a cached copy exists on the proxy.
#define INTERNET_FLAG_NO_CACHE_WRITE    0x04000000 // Does not add the returned entity to the cache. 
#define INTERNET_FLAG_RELOAD            0x80000000 // Forces a download of the requested file, object, or directory listing from the origin server, not from the cache.

int InternetOpenA(
	string 	sAgent,
	int		lAccessType,
	string 	sProxyName="",
	string 	sProxyBypass="",
	int 	lFlags=0
);

int InternetOpenUrlA(
	int 	hInternetSession,
	string 	sUrl, 
	string 	sHeaders="",
	int 	lHeadersLength=0,
	int 	lFlags=0,
	int 	lContext=0 
);

int InternetReadFile(
	int 	hFile,
	string 	sBuffer,
	int 	lNumBytesToRead,
	int& 	lNumberOfBytesRead[]
);

int InternetCloseHandle(
	int 	hInet
);
#import


int hSession(bool Direct)
{
	string InternetAgent;
	if (hSession_IEType == 0)
	{
		InternetAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Q312461)";
		hSession_IEType = InternetOpenA(InternetAgent, Internet_Open_Type_Preconfig, "0", "0", 0);
		hSession_Direct = InternetOpenA(InternetAgent, Internet_Open_Type_Direct, "0", "0", 0);
	}
	if (Direct) 
	{ 
		return(hSession_Direct); 
	}
	else 
	{
		return(hSession_IEType); 
	}
}


bool GrabWeb(string strUrl, string& strWebPage)
{
	int 	   hInternet;
	int	   iResult;
	int 	   lReturn[]	= {1};
	string   sBuffer		= "                                                                                                                                                                                                                                                               ";	// 255 spaces
	int 	   bytes;
	
	hInternet = InternetOpenUrlA(hSession(FALSE), strUrl, "0", 0, 
								INTERNET_FLAG_NO_CACHE_WRITE | 
								INTERNET_FLAG_PRAGMA_NOCACHE | 
								INTERNET_FLAG_RELOAD, 0);
								
	if (bWinInetDebug) 
		Print (NEWS_MODULE_NAME, ": hInternet: " + hInternet);   
	if (hInternet == 0) 
		return(false);

	Print("Reading URL: " + strUrl);	   //added by MN	
	iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
	
	if (bWinInetDebug) 
		Print (NEWS_MODULE_NAME, ": iResult: " + iResult);
	if (bWinInetDebug) 
		Print (NEWS_MODULE_NAME, ": lReturn: " + lReturn[0]);
	if (bWinInetDebug) 
		Print (NEWS_MODULE_NAME, ": iResult: " + iResult);
	if (bWinInetDebug) 
		Print (NEWS_MODULE_NAME, ": sBuffer: " +  sBuffer);
	if (iResult == 0) 
		return(false);
	bytes = lReturn[0];

	strWebPage = StringSubstrOld(sBuffer, 0, lReturn[0]);
	
	// If there's more data then keep reading it into the buffer
	while (lReturn[0] != 0)
	{
		iResult = InternetReadFile(hInternet, sBuffer, Buffer_LEN, lReturn);
		if (lReturn[0]==0) 
			break;
		bytes = bytes + lReturn[0];
		strWebPage = strWebPage + StringSubstrOld(sBuffer, 0, lReturn[0]);
	}

	Print(NEWS_MODULE_NAME, ": Closing URL web connection");   //added by MN
	iResult = InternetCloseHandle(hInternet);
	if (iResult == 0) 
		return(false);
		
	return(true);
}

//=================================================================================================
//=================================================================================================
//===================================   Timezone Functions   ======================================
//=================================================================================================
//=================================================================================================


#import "kernel32.dll"
int  GetTimeZoneInformation(int& TZInfoArray[]);
#import

#define NEWS_TIME_ZONE_ID_UNKNOWN   0
#define NEWS_TIME_ZONE_ID_STANDARD  1
#define NEWS_TIME_ZONE_ID_DAYLIGHT  2

// int TZInfoArray[43];	
//	TZInfoArray[0]: Bias in minutes. UTC = local time + Bias
// TZInfoArray[42]: DaylightBias in minutes. This value is added to the value of the Bias member to form the bias used during daylight saving time.

datetime TimeNEWS_GMT(){ // TimeCurrent to NEWS_GMT
    datetime    serverTime = AltTimeCurrent();
    if (SERVER_To_UTC_Auto)
        ServerNEWS_GMTZone = - (LocalTimeGMT() - serverTime +1800)/3600;
    return (serverTime - ServerNEWS_GMTZone * 3600);
}

datetime LocalTimeGMT(){    // TimeLocal to NEWS_GMT Unk: forum.mql4.com/12057#522900
	int TZInfoArray[43];                                
	#define UNKNOWN  0
	int tz  = GetTimeZoneInformation(TZInfoArray);      
	#define DAYLIGHT 2
	int NEWS_GMTshift    = TZInfoArray[0];   /* GetTimeZoneInformation will return
                                         * the right Bias even when it returns
                                         * UNKNOWN==0 */
	if(tz == DAYLIGHT || tz == UNKNOWN)
		NEWS_GMTshift += TZInfoArray[42];
	if (SERVER_To_UTC_Auto)
		return (AltTimeLocal() + NEWS_GMTshift * 60);
	return (AltTimeLocal() - LocalNEWS_GMTZone * 3600);
}

datetime TimeNEWS_GMTOffset () {
	return (AltTimeLocal() - LocalTimeGMT());
}


datetime AltTimeCurrent() {
	datetime theTimeCurrent = TimeCurrent();

	if(GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
      theTimeCurrent = GlobalVariableGet("_TimeReference");

	return (theTimeCurrent);
}


datetime AltTimeLocal() {
	datetime theTimeLocal = TimeLocal();

	if(GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
      theTimeLocal = GlobalVariableGet("_TimeReference");

	return (theTimeLocal);
}

// TimeLocal() Returns local computer time as number of seconds elapsed from 00:00 January 1, 1970.
// But at the testing, local time is modelled and is the same as the modelled last known server time. 
// This means TimeLocal() = TimeCurrent() when testing, but many times TimeLocal() != TimeCurrent in real.
//=================================================================================================

int 		totalLightSavings = 0;
datetime	lightsavings[40][2];
bool		lightsavingsfileread = false;

int ReadCSVDaylightSavingsFile() {
	string file = "daylightsavings.csv";
	int handle = FileOpen( file, FILE_CSV | FILE_READ, "," );

	if ( handle < 0 ) {
		Print( "Can\'t open ", file, ".. the last error is ", GetLastError() );
		return (0);
	}
  
	totalLightSavings = 0;
	while( !FileIsEnding (handle) ) {
		lightsavings[totalLightSavings][0] = StrToTime(FileReadString (handle));
		lightsavings[totalLightSavings][1] = StrToTime(FileReadString (handle));
		totalLightSavings++;
		// Se suma 1h al periodo indicado.
	}
	FileClose (handle);
	
	lightsavingsfileread = true;
	
	Print ("Loaded ", totalLightSavings, " daylight savig periods");
	Print ("Broker time is ", TimeToStr(AltTimeCurrent(), TIME_MINUTES), "(NEWS_GMT+", ServerNEWS_GMTZone, ") ", TimeToStr(AltTimeCurrent() - TimeCurrentToGMT(AltTimeCurrent()), TIME_MINUTES), " ahead of NEWS_GMT");
	Print ("Local time is ", TimeToStr(AltTimeLocal(), TIME_MINUTES), "(NEWS_GMT+", LocalNEWS_GMTZone, ") ", TimeToStr(AltTimeLocal() - TimeLocalToGMT(AltTimeLocal()), TIME_MINUTES), " ahead of NEWS_GMT");
	
   return (totalLightSavings);
}


datetime TimeCurrentToGMT (datetime currenttime) {
	datetime result = currenttime;
	int index = -1;
	
	if (!lightsavingsfileread)
		ReadCSVDaylightSavingsFile() ;
	
	// We look for the correct year
	//for (int i = 0; i < ArraySize(lightsavings); i++) BART
	for (int i = 0; i < ArrayRange(lightsavings, 0); i++)
		if (TimeYear(lightsavings[i][0]) == TimeYear (currenttime))
			index = i;
	if (index != -1)
		if (currenttime > lightsavings[index][0] && currenttime < lightsavings[index][1])
			result -= 3600;
	result -= ServerNEWS_GMTZone * 3600;
	
	return (result);
}


datetime TimeGMTToCurrentTime (datetime NEWS_GMTtime) {
	datetime result = NEWS_GMTtime;
	int index = -1;

	if (!lightsavingsfileread)
		ReadCSVDaylightSavingsFile() ;

	// We look for the correct year
	//for (int i = 0; i < ArraySize(lightsavings); i++) BART
	for (int i = 0; i < ArrayRange(lightsavings, 0); i++)
		if (TimeYear(lightsavings[i][0]) == TimeYear (NEWS_GMTtime))
			index = i;
			
	if (index != -1)
		if (NEWS_GMTtime > lightsavings[index][0] && NEWS_GMTtime < lightsavings[index][1])
			result += 3600;

	result += ServerNEWS_GMTZone * 3600;
	
	return (result);
}


datetime TimeLocalToGMT (datetime localtime) {
	datetime result = localtime;
	int index = -1;
	
	if (!lightsavingsfileread)
		ReadCSVDaylightSavingsFile() ;
	
	// We look for the correct year
//	for (int i = 0; i < ArraySize(lightsavings); i++) BART
   for (int i = 0; i < ArrayRange(lightsavings, 0); i++)
		if (TimeYear(lightsavings[i][0]) == TimeYear (localtime))
			index = i;
			
	if (index != -1)
		if (localtime > lightsavings[index][0] && localtime < lightsavings[index][1]) {
			result -= 3600;
			// Print ("Lighsavings compensation");
		}

	if (GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
      result -= ServerNEWS_GMTZone * 3600;		
	else
      result -= LocalNEWS_GMTZone * 3600;
	
	return (result);
}


datetime TimeGMTToLocalTime (datetime NEWS_GMTtime) {
	datetime result = NEWS_GMTtime;
	int index = -1;

	if (!lightsavingsfileread)
		ReadCSVDaylightSavingsFile() ;

	// We look for the correct year
	//for (int i = 0; i < ArraySize(lightsavings); i++) BART
	for (int i = 0; i < ArrayRange(lightsavings, 0); i++)
		if (TimeYear(lightsavings[i][0]) == TimeYear (NEWS_GMTtime))
			index = i;
			
	if (index != -1)
		if (NEWS_GMTtime > lightsavings[index][0] && NEWS_GMTtime < lightsavings[index][1])
			result += 3600;

	if (GlobalVariableCheck("_TimeReference") && UseExternalTimeReference)
      result += ServerNEWS_GMTZone * 3600;		
	else
      result += LocalNEWS_GMTZone * 3600;
	
	return (result);
}


//+----------------------------------------------------------------------------+
//|                                                              mql4-http.mqh |
//+----------------------------------------------------------------------------+
//|                                                      Built by Sergey Lukin |
//|                                                    contact@sergeylukin.com |
//|                                                                            |
//| This libarry is highly based on following:                                 |
//|                                                                            |
//| - HTTP Wininet sample: http://codebase.mql4.com/8115                       |
//| - EasyXML parser: http://www.mql5.com/code/1998                            |
//|                                                                            |
//+----------------------------------------------------------------------------+

#import "wininet.dll"
int InternetOpenW(
    string     sAgent,
    int        lAccessType,
    string     sProxyName="",
    string     sProxyBypass="",
    int     lFlags=0
);
int InternetOpenUrlW(
    int     hInternetSession,
    string     sUrl, 
    string     sHeaders="",
    int     lHeadersLength=0,
    int     lFlags=0,
    int     lContext=0 
);
int InternetReadFile(
    int     hFile,
    uchar  &   sBuffer[],
    int     lNumBytesToRead,
    int&     lNumberOfBytesRead
);
int InternetCloseHandle(
    int     hInet
);       
#import

// int hSession_IEType;
// int hSession_Direct;
// int Internet_Open_Type_Preconfig = 0;
// int Internet_Open_Type_Direct = 1;

int hSessionB(bool Direct)
{
    string InternetAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; Q312461)";
    
    if (Direct) 
    { 
        if (hSession_Direct == 0)
        {
            hSession_Direct = InternetOpenW(InternetAgent, Internet_Open_Type_Direct, "0", "0", 0);
        }
        
        return(hSession_Direct); 
    }
    else 
    {
        if (hSession_IEType == 0)
        {
           hSession_IEType = InternetOpenW(InternetAgent, Internet_Open_Type_Preconfig, "0", "0", 0);
        }
        
        return(hSession_IEType); 
    }
}

string httpGET(string strUrl)
{
   int handler = hSessionB(false);
   int response = InternetOpenUrlW(handler, strUrl);
   if (response == 0) 
        return(false);
        
   uchar ch[100]; string toStr=""; int dwBytes, h=-1;
   while(InternetReadFile(response, ch, 100, dwBytes)) 
  {
    if (dwBytes<=0) break; toStr=toStr+CharArrayToString(ch, 0, dwBytes);
  }
  
  InternetCloseHandle(response);
  return toStr;
}

string StringSubstrOld(string x,int a,int b=-1) {

if (a < 0) a= 0; // Stop odd behaviour

if (b<=0) b = -1; // new MQL4 EOL flag

return StringSubstr(x,a,b);

}